import packetGenerator as packet
import communicator
import validation
import time


# ------------------------------------------#
# Fly in the full manual mode
# ------------------------------------------#
def fly_manual_mode(thrust, roll, pitch, yaw_rate):
    """
    Fly in the full manual mode

    Args:
        thrust (int): force produce the propellers. Manually maintain flying height. Range is (0)-(100).
        roll (int): rotation around the x-axis (front-to-back axis). Tilt left or right to move left or right. Range is (-30°)-(+30°).
        pitch (int): rotation around the y-axis (side-to-side axis). Tile front or back to move forward or backward. Range is (-30°)-(+30°).
        yaw_rate (int): rotation around the z-axis. Rotate the drone's heading to left or right. Range is (-100)-(+100)

    Returns:
        bool: validity of entered arguments
    """
    t = validation.thrust(thrust)
    r = validation.euler(roll)
    p = validation.euler(pitch)
    y = validation.yaw_rate(yaw_rate)

    data_valid = t and r and p and y

    if data_valid:
        data_packet = packet.set_manual(thrust, roll, pitch, yaw_rate)
        communicator.transmit(data_packet)
        return 1

    else:
        return 0


# ------------------------------------------#
# Fly in the alt hold mode
# ------------------------------------------#
def fly_altitude_hold_mode(height, roll, pitch, yaw_rate):
    """
    Fly in the altitude hold mode

    Args:
        height (float): flying height to maintain. Range is (0.0m)-(2.0m).
        roll (int): rotation around the x-axis (front-to-back axis). Tilt left or right to move left or right. Range is (-30°)-(+30°).
        pitch (int): rotation around the y-axis (side-to-side axis). Tile front or back to move forward or backward. Range is (-30°)-(+30°).
        yaw_rate (int): rotation around the z-axis. Rotate the drone's heading to left or right. Range is (-100)-(+100).

    Returns:
        bool: validity of entered arguments
    """
    h = validation.height(height)
    r = validation.euler(roll)
    p = validation.euler(pitch)
    y = validation.yaw_rate(yaw_rate)

    data_valid = h and r and p and y

    if data_valid:
        data_packet = packet.alt_hold(height, roll, pitch, yaw_rate)
        communicator.transmit(data_packet)
        return 1

    else:
        return 0


# ------------------------------------------#
# Automatic takeoff
# ------------------------------------------#
def takeoff(height=0.5):
    """
    Automatic takeoff

    Args:
        height (float): flying height to maintain. Default value is 0.5m. Range is (0.0m)-(2.0m)

    Returns:
        Takeoff status
    """
    h = validation.height(height)

    data_valid = h

    if data_valid:
        data_packet = packet.takeoff(height)
        communicator.transmit(data_packet)

        message_confirmed = communicator.get_message_confirmation()

        if message_confirmed:
            print("Takeoff Initialized")  # takeoff command received by the drone

        task_confirmed = communicator.get_action_confirmation(2)

        if task_confirmed:
            print("Takeoff Succeeded")  # takeoff task executed by the drone
            return 1

        else:
            print("Takeoff Failed!")
            return 0

    else:
        return 0

# ------------------------------------------#
# Automatic takeoff and hover for a certain time
# ------------------------------------------#
def takeoff_and_hover(height=0.5, duration = 5):
    """
    Automatic takeoff

    Args:
        height (float): flying height to maintain. Default value is 0.5m. Range is (0.0m)-(2.0m)

    Returns:
        Takeoff status
    """
    h = validation.height(height)

    data_valid = h

    if data_valid:
        data_packet = packet.takeoff(height)
        communicator.transmit(data_packet)

        message_confirmed = communicator.get_message_confirmation()

        if message_confirmed:
            print("Takeoff Initialized")  # takeoff command received by the drone

        start_time = time.time()
        task_confirmed = communicator.get_action_confirmation(2)

        if task_confirmed:
            print("Takeoff Succeeded & start hovering")  # takeoff task executed by the drone

            while time.time() - start_time < duration:
                communicator.transmit(data_packet)
                time.sleep(0.5)

        else:
            print("Takeoff Failed!")
            return 0

    else:
        return 0


# ------------------------------------------#
# Set height
# ------------------------------------------#
def set_height(height=0.5):
    """
    Set/ adjust flying height

    Args:
        height (float): flying height to maintain. Default value is 0.5m. Range is (0.0m)-(2.0m)

    Returns:
        Height changing status
    """
    h = validation.height(height)

    data_valid = h

    if data_valid:
        data_packet = packet.takeoff(height)
        communicator.transmit(data_packet)

        message_confirmed = communicator.get_message_confirmation()

        if message_confirmed:
            print("Height Adjust Initialized")  # takeoff command received by the drone

        task_confirmed = communicator.get_action_confirmation(1)

        if task_confirmed:
            print("Height Adjust Succeeded")  # takeoff task executed by the drone
            return 1

        else:
            print("Height Adjust Failed!")
            return 0

    else:
        return 0


# ------------------------------------------#
# Automatic land
# ------------------------------------------#
def land():
    """
    Land the drone

    Returns:
        Landing status
    """
    data_packet = packet.land()
    communicator.transmit(data_packet)

    message_confirmed = communicator.get_message_confirmation()

    if message_confirmed:
        print("Land Initiated")

    task_confirmed = communicator.get_action_confirmation(1)

    if task_confirmed:
        print("Land Successful")
        return 1
    else:
        print("Land Failed!")
        return 0


# ------------------------------------------#
# Linear movements
# ------------------------------------------#
def movement_constructor(velocity, time_duration, direction):
    """
    A helping function for linear movement functions

    Args:
        velocity (float): moving velocity. Range is (0.0ms⁻¹)-(0.5ms⁻¹). Will be auto limited if range exceeded.
        time_duration (float): flying time for the movement. Range is (0s)-(10s). Will be auto limited if range exceeded.
        direction (char): direction ID

    Returns:
        Movement status
    """
    v = validation.movement_velocity(velocity)

    data_valid = v

    if data_valid:
        # limiting velocity values
        if velocity < 0.0:
            velocity = 0.0
        elif velocity > 0.5:
            velocity = 0.5

        # limiting time duration
        if time_duration < 0.0:
            time_duration = 0.0
        elif time_duration > 10.0:
            time_duration = 10.0

        data_packet = packet.linear_movement(velocity, time_duration, direction)
        communicator.transmit(data_packet)

        message_confirmed = communicator.get_message_confirmation()

        if message_confirmed:
            print(
                direction + " Movement Initiated"
            )  # takeoff command received by the drone

        task_confirmed = communicator.get_action_confirmation(time_duration)

        if task_confirmed:
            print(
                direction + " Movement Successful"
            )  # takeoff task executed by the drone
            return 1

        else:
            print(direction + " Movement Failed!")
            return 0

    else:
        return 0


def go_forward(velocity=0.2, time_duration=1.0):
    """
    Move forward for a given time period with a given velocity

    Args:
        velocity (float): forward moving velocity. Default value is 0.2ms⁻¹. Range is (0.0ms⁻¹)-(0.5ms⁻¹). Will be auto limited if range exceeded.
        time_duration (float): flying time for the movement. Default value is 1s. Range is (0s)-(10s). Will be auto limited if range exceeded.
    """
    movement_constructor(velocity, time_duration, "F")


def go_backward(velocity=0.2, time_duration=1.0):
    """
    Move backward for a given time period with a given velocity

    Args:
        velocity (float): backward moving velocity. Default value is 0.2ms⁻¹. Range is (0.0ms⁻¹)-(0.5ms⁻¹). Will be auto limited if range exceeded.
        time_duration (float): flying time for the movement. Default value is 1s. Range is (0s)-(10s). Will be auto limited if range exceeded.
    """
    movement_constructor(velocity, time_duration, "B")


def go_left(velocity=0.2, time_duration=1.0):
    """
    Move left for a given time period with a given velocity

    Args:
        velocity (float): left moving velocity. Default value is 0.2ms⁻¹. Range is (0.0ms⁻¹)-(0.5ms⁻¹). Will be auto limited if range exceeded.
        time_duration (float): flying time for the movement. Default value is 1s. Range is (0s)-(10s). Will be auto limited if range exceeded.
    """
    movement_constructor(velocity, time_duration, "L")


def go_right(velocity=0.2, time_duration=1.0):
    """
    Move right for a given time period with a given velocity

    Args:
        velocity (float): right moving velocity. Default value is 0.2ms⁻¹. Range is (0.0ms⁻¹)-(0.5ms⁻¹). Will be auto limited if range exceeded.
        time_duration (float): flying time for the movement. Default value is 1s. Range is (0s)-(10s). Will be auto limited if range exceeded.
    """
    movement_constructor(velocity, time_duration, "R")


# ------------------------------------------#
# Rotational movements
# ------------------------------------------#
def rotation_constructor(angle, direction):
    """
    A helping function for rotational movement functions

    Args:
        angle (float): rotation in degrees.
        direction (char): direction ID

    Returns:
        Rotation status
    """
    a = validation.angle(angle)

    data_valid = a

    if data_valid:
        data_packet = packet.rotational_movement(angle, direction)
        communicator.transmit(data_packet)

        message_confirmed = communicator.get_message_confirmation()

        if message_confirmed:
            print(direction + " Rotation Initiated")

        task_confirmed = communicator.get_action_confirmation(2)

        if task_confirmed:
            print(direction + " Rotation Successful")
            return 1

        else:
            print(direction + " Rotation Failed!")
            return 0

    else:
        return 0


def rotate_left(angle):
    """
    Rotate (change heading) to left for given angle

    Args:
        angle (float): rotation in degrees
    """
    rotation_constructor(angle, "L")


def rotate_right(angle):
    """
    Rotate (change heading) to right for given angle

    Args:
        angle (float): rotation in degrees
    """
    rotation_constructor(angle, "R")


# ------------------------------------------#
# Set PID values
# ------------------------------------------#
def set_PID(type, KP, KI, KD):
    """
    Set pid values

    Args:
        type (PID_TYPES): PID type to set
        KP (float): kP value to set for the PID type
        KI (float): kI value to set for the PID type
        KD (float): kD value to set for the PID type

    Returns:
        PID set status
    """
    p = validation.positive_float(KP)
    i = validation.positive_float(KI)
    d = validation.positive_float(KD)
    t = validation.pid_type(type)

    data_valid = p and i and d and t

    if data_valid:
        data_packet = packet.set_PID(type, KP, KI, KD)
        communicator.transmit(data_packet)

        message_confirmed = communicator.get_message_confirmation()

        if message_confirmed:
            print("PID Set Succeeded")
            return 1
        else:
            print("PID Set Failed")
            return 0

    else:
        return 0


# ------------------------------------------#
# Get connection verification using LED
# ------------------------------------------#
def verification_LED():
    dataPacket = packet.verification_LED()
    communicator.transmit(dataPacket)
