import json

CONTROLLER_MODE = {
    "WIFI_MODE_SDK": 1,
}

PACKET_MODES = {
    "MANUAL_MODE": 1,
    "ALT_HOLD_MODE": 2,
    "AUTO_MODE": 3,
    "LAND": 4,
    "TELEMETRY": 5,
    "TUNE_PID": 6,
    "VERIFY_BY_LED": 7,
}


# ------------------------------------------#
# Create packet for the manual mode
# ------------------------------------------#
def set_manual(thrust, roll, pitch, yaw_rate):
    """
    Create data packet for full manual mode

    Args:
        thrust (int): force produce the propellers. Manually maintain flying height. Range is (0)-(100).
        roll (int): rotation around the x-axis (front-to-back axis). Tilt left or right to move left or right. Range is (-30°)-(+30°).
        pitch (int): rotation around the y-axis (side-to-side axis). Tile front or back to move forward or backward. Range is (-30°)-(+30°).
        yaw_rate (int): rotation around the z-axis. Rotate the drone's heading to left or right. Range is (-100)-(+100)

    Returns:
        UDP data packet
    """
    packet = {
        "WM": CONTROLLER_MODE["WIFI_MODE_SDK"],
        "M": PACKET_MODES["MANUAL_MODE"],
        "T": thrust,
        "R": roll,
        "P": pitch,
        "Y": yaw_rate,
    }

    packet = json.dumps(packet)
    return packet.encode()


# ------------------------------------------#
# Create packet for the altitude hold mode
# ------------------------------------------#
def alt_hold(height, roll, pitch, yaw_rate):
    """
    Create packet for the altitude hold mode

    Args:
        height (float): flying height to maintain. Range is (0.0m)-(2.0m).
        roll (int): rotation around the x-axis (front-to-back axis). Tilt left or right to move left or right. Range is (-30°)-(+30°).
        pitch (int): rotation around the y-axis (side-to-side axis). Tile front or back to move forward or backward. Range is (-30°)-(+30°).
        yaw_rate (int): rotation around the z-axis. Rotate the drone's heading to left or right. Range is (-100)-(+100).

    Returns:
        UDP data packet
    """
    packet = {
        "WM": CONTROLLER_MODE["WIFI_MODE_SDK"],
        "M": PACKET_MODES["ALT_HOLD_MODE"],
        "H": round(height, 1),
        "R": round(roll),
        "P": round(pitch),
        "Y": round(yaw_rate),
    }

    packet = json.dumps(packet)
    return packet.encode()


# ------------------------------------------#
# Create packet for the position hold mode
# ------------------------------------------#
def pos_hold(height, vx, vy, yaw_rate):
    """
    Create packet for the position hold mode

    Args:
        height (float): flying height to maintain. Range is (0.0m)-(2.0m).
        vx (): TODO add
        vy (): TODO add
        yaw_rate (int): rotation around the z-axis. Rotate the drone's heading to left or right. Range is (-100)-(+100).

    Returns:
        UDP data packet
    """
    packet = {
        "WM": CONTROLLER_MODE["WIFI_MODE_SDK"],
        "M": PACKET_MODES["AUTO_MODE"],
        # "Command":1,
        "H": height,
        "VX": vx,
        "VY": vy,
        "Y": yaw_rate,
    }

    packet = json.dumps(packet)
    return packet.encode()


# ------------------------------------------#
# Create packet for the linear movements
# ------------------------------------------#
def linear_movement(set_vel, duration, direction):
    """
    Create packet for the linear movements

    Args:
        velocity (float): moving velocity. Range is (0.0ms⁻¹)-(0.5ms⁻¹). Will be auto limited if range exceeded.
        time_duration (float): flying time for the movement. Range is (0s)-(10s). Will be auto limited if range exceeded.
        direction (char): direction ID

    Returns:
        UDP data packet
    """
    packet = {
        "WM": CONTROLLER_MODE["WIFI_MODE_SDK"],
        "M": PACKET_MODES["AUTO_MODE"],
        "Command": 3,
        "Dir": direction,
        "Vel": set_vel,
        "Time": duration,
    }

    packet = json.dumps(packet)
    return packet.encode()


# ------------------------------------------#
# Create packet for the rotational movements
# ------------------------------------------#
def rotational_movement(angle, direction):
    """
    Create packet for the rotational movements

    Args:
        angle (float): rotation in degrees.
        direction (char): direction ID

    Returns:
        UDP data packet
    """
    packet = {
        "WM": CONTROLLER_MODE["WIFI_MODE_SDK"],
        "M": PACKET_MODES["AUTO_MODE"],
        "Command": 4,
        "Dir": direction,
        "Angle": angle,
    }

    packet = json.dumps(packet)
    return packet.encode()


# ------------------------------------------#
# Create packet for auto takeoff
# ------------------------------------------#
def takeoff(height=0.5):
    """
    Create packet for auto takeoff

    Args:
        height (float): flying height to maintain. Default value is 0.5m. Range is (0.0m)-(2.0m)

    Returns:
        UDP data packet
    """
    packet = {
        "WM": CONTROLLER_MODE["WIFI_MODE_SDK"],
        "M": PACKET_MODES["AUTO_MODE"],
        "Command": 1,
        "H": height,
    }
    packet = json.dumps(packet)
    return packet.encode()


# ------------------------------------------#
# Create packet for auto takeoff
# ------------------------------------------#
def land():
    """
    Create packet for auto takeoff

    Returns:
        UDP data packet
    """
    packet = {
        "WM": CONTROLLER_MODE["WIFI_MODE_SDK"],
        "M": PACKET_MODES["AUTO_MODE"],
        "Command": 2,
    }

    packet = json.dumps(packet)
    return packet.encode()


# ------------------------------------------#
# Create packet for auto land
# ------------------------------------------#
def set_PID(ID, KP, KI, KD):
    """
    Create packet for auto land

    Args:
        type (PID_TYPES): PID type to set
        KP (float): kP value to set for the PID type
        KI (float): kI value to set for the PID type
        KD (float): kD value to set for the PID type

    Returns:
        UDP data packet
    """
    packet = {
        "WM": CONTROLLER_MODE["WIFI_MODE_SDK"],
        "M": PACKET_MODES["TUNE_PID"],
        "ID": ID,
        "KP": KP,
        "KI": KI,
        "KD": KD,
    }

    packet = json.dumps(packet)
    return packet.encode()


# ------------------------------------------#
# Create packet for LED verification
# ------------------------------------------#
def verification_LED():
    """
    Create packet for LED verification

    Returns:
        UDP data packet

    """
    packet = {
        "WM": CONTROLLER_MODE["WIFI_MODE_SDK"],
        "M": PACKET_MODES["VERIFY_BY_LED"],
    }

    packet = json.dumps(packet)
    return packet.encode()
